# Copyright 2011 by Andrew Scullion

# Permission to use, copy, modify, and distribute this software for
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

# Name :          MCExport
# Description :   Transform a solid into little cubes
# Author :        TheDro
# Usage :         Select a group which contains a solid (no faces inside
# 								the solid for proper behavior) and run the script. It will
# 								run the voxelize script and export the generated geometry
# 								to a .schematic file to be imported into minecraft.
# Requirements :  The Voxelize version 0.4 or above is required to run this
#                 script.
# Warning :       The process is still very slow and may cause Sketchup to
# 								"not respond" even though the script is running in the 
# 								background. Apply this to relatively simple geometry
#									until future releases.
# Date :          2011/04/02
# History:
#		beta 01 : First released version. 
#             This still needs some refining and more thorough commenting.








require 'sketchup'
# Trapping whether a file can be found:
begin
  require 'zlib'
rescue LoadError
  load 'TheDro/tdloadpaths.rb' 
  # try it again
  begin
    require 'zlib'
  rescue LoadError => e
    # load a copy of zlib.so that you distro
    # with your plugin as a backup, ie:
    if e.message =~ /(zlib.so)/
      require 'TheDro/zlib.so'
    else
      raise # re-raise the LoadError
      # as it may be another file that
      # zlib.so required internally.
    end
  end
end


begin
require 'TheDro/voxelizebeta04'
rescue LoadError
	begin
		require 'voxelizebeta04' # In case voxelize is not within the TheDro folder
	rescue LoadError
		UI.messagebox("Voxelize version 0.4 or above is required to load.")
		raise
	end
end


module TheDro::MCExport

	# Thanks to Dan Rathbun for tidying this up!
 unless file_loaded?( File.basename(__FILE__).downcase )

   ###  CONSTANTS
   #
   VERSION = '0.1.0'

   TOPMENU='Plugins'
   SUBMENU=''
   MENUTEXT="MCExport (#{VERSION})"

   ###  MODULE VARS
   #
   @@topmenu = UI.menu(TOPMENU)

   unless SUBMENU.empty?
     @@menu = @@topmenu.add_submenu(SUBMENU)
   else
     @@menu = @@topmenu
   end

   @@item = false
   @@validation = MF_ENABLED

   @@filename = 'c:/'

   # @@op = 0  ### 0.3.0 added

 end


  class << self

    ### ----------------------------------------------------------------- ###
    #
    #  mcexport()
    #
    #  Export voxelized solid to .schematic file
    #
    def mcexport

     while true
       @@filename = UI.openpanel("Choose a .schematic output file",
         @@filename, "*.schematic")

       return if @@filename == nil

       if File.exist?(@@filename) then
         result = UI.messagebox("Overwrite existing file?", MB_YESNO)
         next if result == 7 # No
         break if result == 6 # Yes
       end
       break
     end
			
      ### OPERATION

      binCubes = TheDro::Voxelize.voxelize(false)
			return if binCubes == nil
			binCubes = binCubes[0]

      n = [binCubes.length, binCubes[0].length, binCubes[0][0].length]


      n3 = n[0]*n[1]*n[2]
      binCubes1D = Array.new(n3)

      # Probably not the most efficient way of linearizing the 3D array
      for ii in 0..(n[0]-1)
        for jj in 0..(n[1]-1)
          for kk in 0..(n[2]-1)
						if binCubes[ii][jj][kk] == 1
							# For now, the exporter only makes dirt.
							binCubes1D[ii+n[0]*(jj+n[1]*kk)] = 3
						else
							binCubes1D[ii+n[0]*(jj+n[1]*kk)] = 0
						end
          end
        end
      end
#      puts binCubes.inspect
      # puts binCubes1D.inspect

      outputArray = [10,"Schematic",
        [[2,"Height",n[2]],[2,"Length",n[1]],[2,"Width",n[0]],
        [9,"Entities",10,0,0],[9,"TileEntities",10,0,0],
        [8,"Materials","Alpha"],
        [7,"Blocks",n3,binCubes1D],
        [7,"Data",n3,Array.new(n3,0)]],0]
			
			# puts outputArray.inspect	
			
      outputString = nbtConvert(outputArray)
			
			# puts outputString.inspect
      # @@filename = 'C:\Users\TheDro\Desktop\tester.schematic'
      gzfile = Zlib::GzipWriter.open(@@filename)
      # gzfile = Zlib::GzipWriter.new(f)
      gzfile.write outputString
      gzfile.close
			# puts outputString.inspect
			
			# puts "read test:"
			g = File.open(@@filename,'r')
			# gz = Zlib::GzipReader.new(f)
			# puts gz.read.inspect
			# gz.close
			# puts g.read.inspect
			g.close
			

    end # def mcexport()

    ### ----------------------------------------------------------------- ###
    #
    #  nbtConvert()
    #
    #  Convert array to nambed binary tag string
    #
    def nbtConvert(a)
      s = ""

      begin

        s << a[0,1].pack("c")
        s << [a[1].length].pack("n") << a[1]
        case a[0]
        when nil
          return ""
       # when 0 #TAG_End
         # return s
        when 2 #TAG_Short
          s << a[2,1].pack("n")
        when 7 #TAG_Byte_Array
          length = a[2]
          s << [length].pack("N")
          s << a[3].pack("c#{length}")
        when 8 #TAG_String
          s << [a[2].length].pack("n") << a[2]
        when 9 #TAG_List
          tag = a[2]
          s << [tag].pack("c")
          length = a[3]
          s << [length].pack("n")
          s << a[4,1].pack("n") # This part is not fully implemented
        when 10 #TAG_Compound
          a[2].each{|ii| s << nbtConvert(ii)}
          s << a[3,1].pack("c") # Technically there is a TAG_End here
        end

      rescue e
        puts("\nMCExport Operation Error: #<#{e.class.name}: #{e.message}>\n")
        puts(e.backtrace)
        return nil
      else
        return s
      end
    end # def nbtConvert


		
    ### ----------------------------------------------------------------- ###
    #
    #  use()
    #
    def use()
      @@validation = MF_CHECKED | MF_GRAYED
      mcexport()
      @@validation = MF_ENABLED
    end


  end # class << self


  ### ------------------------------------------------------------------- ###
  #
  #  MENU &| COMMAND
  #
	unless file_loaded?( File.basename(__FILE__).downcase )

		@@item = @@menu.add_item( MENUTEXT ) { use() }

		@@menu.set_validation_proc(@@item) { @@validation }

		file_loaded( File.basename(__FILE__).downcase )

	end


end # module TheDro::Voxelize





